unit Unit1;
{
Multiple Regression is used to make predictions about multiple values.
Use of multiple regression involves the discovering of the relationship between
the values and then finding an equation that satisfies that relationship.

The multiple regression equation takes the form Y = b0+ b1x1 + b2x2 + ... + bnxn .
The b's are the regression coefficients, representing the amount the
dependent variable Y changes when the independent changes 1 unit.
The b0 is the constant, where the regression line intercepts the y axis,
representing the amount the dependent Y will be when all the independent variables are 0.

It is possible to create an unusable equation with the Method of Multiple Regression. The easiest way (although not the only way) is to enter an equation whose complexity is greater than the number of points entered minus 1. 

Predicted values Y`, also called fitted values, are the values of each case based
on using the regression equation for all cases in the analysis.

This demo program calculates the coefficents using least square fitting .
It uses these coefficents to calculate the fitted values Y`.

The correlation coefficient, a quantity which gives the quality of the
least squares fitting to the original data, is also calculated.
It varies from 0 (no relationship) to 1 (perfect linear relationship)
or -1 (perfect negative linear relationship).}



interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, Buttons, StdCtrls, Parser10,LeastSquareFit,StrUtils;

type
  TForm1 = class(TForm)
    Tabelle: TStringGrid;
    GroupBox1: TGroupBox;
    Edit1: TEdit;
    SpeedButton1: TSpeedButton;
    GroupBox2: TGroupBox;
    Edit3: TEdit;
    GroupBox3: TGroupBox;
    Edit2: TEdit;
    Label1: TLabel;
    GroupBox4: TGroupBox;
    Memo1: TMemo;
    GroupBox5: TGroupBox;
    Label2: TLabel;
    procedure SpeedButton1Click(Sender: TObject);
    procedure TabelleDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure FormCreate(Sender: TObject);
    procedure Edit2Change(Sender: TObject);
    procedure Edit3Change(Sender: TObject);
  private
    procedure SetRandomValues;
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

type TVector=array of Extended;

function Calc(Formula:String;const Matrix:TMatrix):TVector;
var DynaParser: TParser;
    k,i:integer;
begin
  SetLength(Result,high(matrix[0])+1);
  DynaParser := TParser.Create(nil);
  for k:=0 to high(matrix)-1 do  //set values to all independend vars
         DynaParser.Variable['x'+inttostr(k+1)] := Matrix[k,0];
  DynaParser.Expression:=copy(Formula,4,Length(Formula)-3);
  for i:=0 to high(matrix[0]) do     // add all samples
  begin
    for k:=0 to high(matrix)-1 do  //set values to all independend vars
         DynaParser.Variable['x'+inttostr(k+1)] := Matrix[k,i];
    Result[i] :=DynaParser.Value;
  end;
  DynaParser.Free;
end;
function LinFit(Formula:String;const Matrix:TMatrix):String;
var DynaParser: TParser;
    i,j,z,k:integer;
    LSF:TLeastSquareFit;
    coef:array of extended; // Array for resulting coefficients
    v: array of extended;
    formulas:array of string;
    success:boolean;
  Procedure SetFormula(Formula:String;index:Integer);
  var k,j:integer;
  begin
    SetLength(formulas,index);
    k:=Length(Formula);
    j:=pos('*',Formula);
    if j=0 then formulas[index-1]:='1' else
    formulas[index-1]:=copy(Formula,j+1,k-j);
  end;
  function splitFormula(s1:String):integer;
  var parenthesis,z,e,i:integer;
      c:char;
  begin
    parenthesis:=0;z:=1;e:=1;
    for i:=1 to length(s1)do begin
     c:=s1[i];
     if (c='+') and (parenthesis=0) then begin
      SetFormula(copy(s1,e,i-e),z);
      e:=i+1;inc(z);
     end;
     if c='(' then inc(parenthesis);
     if c=')' then dec(parenthesis);
    end;
    SetFormula(copy(s1,e,length(s1)-e+1),z);
    result:=z;
  end;
begin
  DynaParser := TParser.Create(nil);
  z:=splitFormula(Formula);
  SetLength(coef,z);                 // vector or resulting coefficients
  SetLength(v, z+1);
  LSF:=TLeastSquareFit.Create(z);
  for i:=0 to high(matrix[0]) do     // for all samples
  begin
    for j:=0 to z-1 do               // for all parts
    begin
      for k:=0 to high(matrix)-1 do  //set values to all independend vars
         DynaParser.Variable['x'+inttostr(k+1)] := Matrix[k,i];
      DynaParser.Expression:=formulas[j];  //set a formular
      v[j] :=DynaParser.Value;       // get calculated value of independend vars
    end;
    v[z] :=Matrix[high(matrix),i];   // value to dependent var
    LSF.AddValue(v);
  end;
  success:=LSF.GetResult(coef);
  LSF.Free;
  DynaParser.Free;
  if success then
  begin
    result:='Y`='+FloatToStr(coef[0]);
    for i:=1 to high(coef) do
    begin
      if coef[i]>=0 then result:=result+'+';
      result:=result+FloatToStr(coef[i])+'*'+formulas[i];
    end;
  end else Result:='';
end;

{------------------------------ Correlation-----------------------------------
Correlation calculates the degree to which two quantities are linearly associated.}
function Correlation(const Matrix:TMatrix):extended;
var c1,c2,i,FNRow:integer;
var xm,ym,x,y,sxx,syy,sxy:extended;
begin
 x:=0;y:=0;FNRow:=High(Matrix[0])+1;
 for i:=0 to FNRow-1 do begin
   y:=y+Matrix[0,i];
   x:=x+Matrix[1,i];;
 end;
 xm:=x/FNRow;ym:=y/FNRow;
 sxx:=0;syy:=0;sxy:=0;
 for i:=0 to FNRow-1  do begin
   y:=Matrix[0,i];
   x:=Matrix[1,i];;
   sxx:=sxx+(x-xm)*(x-xm);
   syy:=syy+(y-ym)*(y-ym);
   sxy:=sxy+(x-xm)*(y-ym);
 end;
 if (sxx<>0) and (syy<>0) then
 begin
   result:=sxy/sqrt(abs(sxx*syy));
  end else result:=0;
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
var Matrix,M2:TMatrix;
    Expectation:TVector;
    i,j,Cols:integer;
begin
  for Cols:=1 to tabelle.ColCount-1 do
  begin
    if tabelle.Cells[Cols,0][1]<>'X' then break;
  end;
  SetLength(Matrix,Cols,tabelle.RowCount-1); Randomize;
  for i:=1 to tabelle.RowCount-1 do
  for j:=1 to Cols do
  begin
    if not TryStrtoFloat(tabelle.Cells[j,i],matrix[j-1,i-1]) then
    begin
      ShowMessage('not a number in sample '+inttostr(i));
      tabelle.Col:=j;
      tabelle.Row:=i;
      exit;
    end;
  end;
  Memo1.Text :=LinFit(edit1.Text,Matrix);
  Expectation:= Calc(Memo1.Text,Matrix);
  SetLength(M2,2,tabelle.RowCount-1);
  for i:=1 to tabelle.RowCount-1 do
  begin
    tabelle.Cells[tabelle.ColCount-1,i]:=floatToStr(Expectation[i-1]);
    M2[0,i-1]:=Matrix[Cols-1,i-1];
    M2[1,i-1]:=Expectation[i-1];
  end;
  label2.Caption:=FloatToStr(Correlation(M2));
end;
procedure TForm1.SetRandomValues;
var i,j,Cols:integer;
begin
  for Cols:=1 to tabelle.ColCount-1 do
  begin
    if tabelle.Cells[Cols,0][1]<>'X' then break;
  end;
  Randomize;
  for i:=1 to tabelle.RowCount-1 do
  for j:=1 to Cols do
  begin
    tabelle.Cells[j,i]:=floattostr(i+j+Random(3));
  end;
end;
procedure TForm1.TabelleDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var X,Y:LongInt; R:Trect;
    ColSelect:array[1..20] of boolean;
const ColorIndex :array[1..10] of TColor = (clred,clblue,clgreen,clWhite,
                  clPurple,clYellow,clMaroon,clLime,clFuchsia,clBlack);
begin
 Tabelle.DefaultDrawing:=true;
 if (Acol in [1..high(ColSelect)]) and (Arow=0) then
  begin
    X:=Rect.Left+2 * (Rect.Right-Rect.Left) Div 3 -0;
    Y:=Rect.Bottom+(((Rect.Top-Rect.Bottom)-17) Div 2);
    Tabelle.Canvas.Pen.Color := clblack;
    Tabelle.Canvas.Brush.Color:=clgreen;
    R.Left:=x; R.Right:=x+10;R.Top:=y+4;R.Bottom:=y+14;
    if ColSelect[ACol] then
      Tabelle.Canvas.Brush.Color := ColorIndex[((ACol-1) mod 10) +1] else
      Tabelle.Canvas.Brush.Color := clGray;
    Tabelle.Canvas.Ellipse(x-1, y+4, x+8, y+13);
    Tabelle.Canvas.Brush.Color := clblack;
  end else inherited;
end;

procedure TForm1.FormCreate(Sender: TObject);
var i:integer;
begin
  DecimalSeparator:='.'; //global
  tabelle.RowCount:=10;
  tabelle.ColCount:=5;
  for i:=1 to tabelle.RowCount-1 do
  begin
    tabelle.Cells[0,i] := inttostr(i);
  end;
  tabelle.Cells[tabelle.ColCount-1,0]:=' Y ';
  for i:=1 to tabelle.ColCount-2 do
  begin
    tabelle.Cells[i,0]:='X'+Chr(48+i);
 end;
 tabelle.ColCount:=tabelle.ColCount+1;
 tabelle.Cells[tabelle.ColCount-1,0]:=' Y` ';
 edit2.Text:=inttostr(tabelle.RowCount-1);
 edit3.Text:=inttostr(tabelle.ColCount-2);
 SetRandomValues;
end;

procedure TForm1.Edit2Change(Sender: TObject);
var i:integer;
begin
  if tryStrToInt(edit2.Text,i) then
  tabelle.RowCount:=i+1;
  for i:=1 to tabelle.RowCount-1 do
  begin
    tabelle.Cells[0,i] := inttostr(i);
  end;
end;

procedure TForm1.Edit3Change(Sender: TObject);
var i:integer;
begin
  if tryStrToInt(edit3.Text,i) then
  tabelle.ColCount:=i+2;
  tabelle.Cells[tabelle.ColCount-1,0]:=' Y ';
  for i:=1 to tabelle.ColCount-2 do
  begin
    tabelle.Cells[i,0]:='X'+Chr(48+i);
  end;
  tabelle.ColCount:=tabelle.ColCount+1;
  tabelle.Cells[tabelle.ColCount-1,0]:=' Y` ';
end;

end.
